# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, \
	RESPONSE_OK, VBox, CheckButton, Alignment, Frame, SHADOW_NONE, Label, SpinButton, Adjustment, HBox, \
	RadioButton, Notebook, STOCK_GO_UP, STOCK_GO_DOWN, EXPAND, FILL, ListStore, TreeView, CellRendererText, \
	ScrolledWindow, POLICY_AUTOMATIC, SHADOW_IN, Alignment, Frame, SHADOW_NONE, Table, Entry, Label, \
	VButtonBox, BUTTONBOX_START, Button, STOCK_ADD, STOCK_DELETE, STOCK_EDIT, accelerator_parse, MessageDialog

from Settings import Settings
from util import str_bool
from ProfileDialog import ProfileDialog


class ConfigureDialog(Dialog):
	def __init__(self):
		Dialog.__init__(self, "Configure LaTeX Plugin", None, DIALOG_MODAL|DIALOG_DESTROY_WITH_PARENT, 
					    (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		
		self.set_border_width(5)
		self.set_has_separator(False)
		
		self.vbox.set_spacing(5)
		
		self.__profile_dialog = None

		self.__settings = Settings()
		
		
		# general
		lbl_toolbar = Label("<b>Toolbar</b>")
		lbl_toolbar.set_use_markup(True)
		lbl_toolbar.set_alignment(0, .5)
		
		self._rad_toolbar_enable = RadioButton(None, "Always _visible")
		self._rad_toolbar_auto = RadioButton(self._rad_toolbar_enable, "Only visible when _editing LaTeX")
		self._rad_toolbar_disable = RadioButton(self._rad_toolbar_enable, "_Disabled")
		
		mode = self.__settings.setdefault("ToolbarMode", "auto")
		if mode == "on":
			self._rad_toolbar_enable.set_active(True)
		elif mode == "auto":
			self._rad_toolbar_auto.set_active(True)
		elif mode == "off":
			self._rad_toolbar_disable.set_active(True)
		
		tbl_general = Table()
		tbl_general.set_border_width(10)
		#tbl_general.set_row_spacings(5)
		tbl_general.set_col_spacings(5)
		tbl_general.attach(lbl_toolbar, 0, 2, 0, 1)
		tbl_general.attach(self._rad_toolbar_enable, 1, 2, 1, 2)
		tbl_general.attach(self._rad_toolbar_auto, 1, 2, 2, 3)
		tbl_general.attach(self._rad_toolbar_disable, 1, 2, 3, 4)
		tbl_general.set_col_spacing(0, 20)
		
		box_general = VBox()
		box_general.pack_start(tbl_general, False)
		
		
		# build profiles
		self.store_profiles = ListStore(str, object) 		# markup, profile
		
		self.view_profiles = TreeView(self.store_profiles)
		self.view_profiles.set_headers_visible(False)
		self.view_profiles.insert_column_with_attributes(-1, "", CellRendererText(), markup=0)
		self.view_profiles.connect("row-activated", lambda *x: self.__configure_profile())
		
		scr_profiles = ScrolledWindow()
		scr_profiles.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
		scr_profiles.set_shadow_type(SHADOW_IN)
		scr_profiles.add(self.view_profiles)
		
		btn_add_profile = Button(stock=STOCK_ADD)
		btn_add_profile.connect("clicked", lambda x: self.__add_profile())
		
		btn_delete_profile = Button(stock=STOCK_DELETE)
		btn_delete_profile.connect("clicked", lambda x: self.__delete_profile())
		
		btn_edit_profile = Button(stock=STOCK_EDIT)
		btn_edit_profile.connect("clicked", lambda x: self.__configure_profile())
		
		buttons = VButtonBox()
		buttons.set_layout(BUTTONBOX_START)
		buttons.add(btn_add_profile)
		buttons.add(btn_edit_profile)
		buttons.add(btn_delete_profile)
		
		box_profiles = HBox()
		box_profiles.set_border_width(10)
		box_profiles.set_spacing(5)
		box_profiles.add(scr_profiles)
		box_profiles.pack_start(buttons, False)
		
		self.__load_profiles()
		
		
		# code comletion
		self.__chk_immediate = CheckButton("Enable _prompt completion on Ctrl+Space", True)
		self.__chk_immediate.set_active(str_bool(self.__settings.setdefault("PromptCompletion", True)))
		
		self.__chk_delayed = CheckButton("Enable _delayed completion", True)
		self.__chk_delayed.set_active(str_bool(self.__settings.setdefault("DelayedCompletion", True)))
		self.__chk_delayed.connect("toggled", self.__delayed_toggled)
		
		self.__lbl_chars = Label("Triggering characters:")
		self.__lbl_chars.set_alignment(0, .5)
		
		self.__chk_backslash = CheckButton("\\")
		self.__chk_backslash.set_active(str_bool(self.__settings.setdefault("TriggerBackslash", True)))
		
		self.__chk_curly = CheckButton("{")
		self.__chk_curly.set_active(str_bool(self.__settings.setdefault("TriggerBraceLeft", True)))
		
		self.__chk_square = CheckButton("[")
		self.__chk_square.set_active(str_bool(self.__settings.setdefault("TriggerBracketLeft", True)))
		
		self.__chk_round = CheckButton("(")
		self.__chk_round.set_active(str_bool(self.__settings.setdefault("TriggerParenLeft", True)))
		
		self.__lbl_delay = Label("Delay [ms]:")
		self.__lbl_delay.set_alignment(0, .5)
		
		_delay = int(self.__settings.setdefault("CompletionDelay", 500))
		self.__spn_delay = SpinButton(Adjustment(_delay, 0, 5000, 1)) 
		
		tbl_cc = Table()
		
		tbl_cc.attach(self.__chk_immediate, 0, 6, 0, 1)
		tbl_cc.attach(self.__chk_delayed, 0, 6, 1, 2)
		tbl_cc.attach(self.__lbl_chars, 1, 2, 2, 3)
		tbl_cc.attach(self.__chk_backslash, 2, 3, 2, 3)
		tbl_cc.attach(self.__chk_curly, 3, 4, 2, 3)
		tbl_cc.attach(self.__chk_square, 4, 5, 2, 3)
		tbl_cc.attach(self.__chk_round, 5, 6, 2, 3)
		tbl_cc.attach(self.__lbl_delay, 1, 2, 3, 4)
		tbl_cc.attach(self.__spn_delay, 2, 6, 3, 4)
		
		tbl_cc.set_border_width(10)
		tbl_cc.set_row_spacings(5)
		tbl_cc.set_col_spacings(5)
		tbl_cc.set_col_spacing(0, 20)
		
		box_cc = VBox()
		box_cc.pack_start(tbl_cc, False)
		
		# notebook
		notebook = Notebook()
		notebook.set_border_width(5)
		notebook.append_page(box_general, Label("General"))
		notebook.append_page(box_profiles, Label("Build Profiles"))
		notebook.append_page(box_cc, Label("Code Completion"))
		
		self.vbox.add(notebook)
		
		self.connect("response", self.__on_response)
		self.set_default_response(RESPONSE_OK)
		
		self.set_size_request(400, 300)
		
		self.show_all()
	
	def __on_response(self, dialog, response):
		if response == RESPONSE_OK:
			# store settings
			self.__settings["PromptCompletion"] = self.__chk_immediate.get_active()
			self.__settings["DelayedCompletion"] = self.__chk_delayed.get_active()
			self.__settings["TriggerBackslash"] = self.__chk_backslash.get_active()
			self.__settings["TriggerBracketLeft"] = self.__chk_square.get_active()
			self.__settings["TriggerBraceLeft"] = self.__chk_curly.get_active()
			self.__settings["TriggerParenLeft"] = self.__chk_round.get_active()
			self.__settings["CompletionDelay"] = self.__spn_delay.get_value_as_int()
			
			if self._rad_toolbar_enable.get_active():
				self.__settings["ToolbarMode"] = "on"
			elif self._rad_toolbar_auto.get_active():
				self.__settings["ToolbarMode"] = "auto"
			elif self._rad_toolbar_disable.get_active():
				self.__settings["ToolbarMode"] = "off"
			
		self.destroy()
	
	def __delayed_toggled(self, togglebutton):
		value = togglebutton.get_active()
		
		self.__chk_backslash.set_sensitive(value)
		self.__chk_curly.set_sensitive(value)
		self.__chk_round.set_sensitive(value)
		self.__chk_square.set_sensitive(value)
		self.__lbl_chars.set_sensitive(value)
		self.__lbl_delay.set_sensitive(value)
		self.__spn_delay.set_sensitive(value)
	
	def __load_profiles(self):
		""" 
		Loads all profiles from Settings into the ListStore
		"""
		self.store_profiles.clear()
		for profile in self.__settings.list_profiles():
			self.store_profiles.append([profile.get_markup(), profile])
	
	def __add_profile(self):
		"""
		Add a new profile
		"""
		if not self.__profile_dialog:
			self.__profile_dialog = ProfileDialog(self)
		
		self.__profile_dialog.set_profile(None)
		
		if self.__profile_dialog.run() == RESPONSE_OK:
			self.__settings.save_profile(self.__profile_dialog.get_profile())
			self.__load_profiles()
			
		self.__profile_dialog.hide()
	
	def __configure_profile(self):
		""" 
		Configure the selected profile 
		"""
		list_store, list_iter = self.view_profiles.get_selection().get_selected()
		profile = list_store.get_value(list_iter, 1)
		
		if not self.__profile_dialog:
			self.__profile_dialog = ProfileDialog(self)
		
		self.__profile_dialog.set_profile(profile)
		
		if self.__profile_dialog.run() == RESPONSE_OK:
			p = self.__profile_dialog.get_profile()
			self.__settings.update_profile(p)
			self.__load_profiles()
		
		self.__profile_dialog.hide()
	
	def __delete_profile(self):
		"""
		Delete the selected profile
		"""
		store, it = self.view_profiles.get_selection().get_selected()
		if it:
			profile = store.get_value(it, 1)
			self.__settings.delete_profile(profile)
			store.remove(it)
			
