# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import ScrolledWindow, TextView, TextBuffer, POLICY_AUTOMATIC
from pango import FontDescription, WEIGHT_BOLD
from time import strftime

from Logger import debug

class ConsoleView(ScrolledWindow):
	def __init__(self):
		debug(self, "__init__")
		
		ScrolledWindow.__init__(self)
		
		self.__ui_built = False
		self.__handler_expose = self.connect("expose-event", self.__exposed)
		
		debug(self, "__init__", "done")
	
	def __exposed(self, widget, event):
		debug(self, "__exposed")
		
		self.__assert_ui()
		self.disconnect(self.__handler_expose)
	
	def __assert_ui(self):
		"""
		Assert that the UI is completely built
		"""
		if not self.__ui_built:
			self.__buffer = TextBuffer()
			self.__buffer.create_tag("timestamp", foreground="darkgrey")
			self.__buffer.create_tag("command", weight=WEIGHT_BOLD)
			
			self.__view = TextView(self.__buffer)
			self.__view.set_editable(False)
			self.__view.modify_font(FontDescription("Monospace"))
			
			self.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
			self.add(self.__view)
			
			self.show_all()
			
			self.__ui_built = True
	
	def clear(self):
		self.__assert_ui()
			
		self.__buffer.set_text("")
	
	def append(self, text, is_command=False):
		"""
		Append text to the console output
		"""
		self.__assert_ui()
		
		# insert timestamp
		timestamp = "[%s] " % strftime("%H:%M:%S")
		
		it_insert = self.__buffer.get_iter_at_mark(self.__buffer.get_insert())
		self.__buffer.insert_with_tags_by_name(it_insert, timestamp, "timestamp")
		
		if is_command:
			# insert command
			it_insert = self.__buffer.get_iter_at_mark(self.__buffer.get_insert())
			self.__buffer.insert_with_tags_by_name(it_insert, text, "command")
		else:
			# insert output
			self.__buffer.insert_at_cursor(text)
		
		self.__buffer.insert_at_cursor("\n")
		
		# scroll to end
		self.__view.scroll_to_iter(self.__buffer.get_end_iter(), 0.0)
		
