# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from os import makedirs
from os.path import expanduser, exists
from shutil import copyfile

from Logger import error, debug

#path_system = "/usr/lib/gedit-2/plugins/LaTeXPlugin"
#path_user = expanduser("~/.gnome2/gedit/plugins/LaTeXPlugin")
#
#if exists(path_system):
#    debug(None, "", "Detected system-wide installation")
#    install_path = path_system
#    
#    if not exists(path_user):
#        # copy settings.xml to ~/.gnome2/gedit/plugins/LaTeXPlugin/
#        makedirs(path_user)
#        
#        try:
#            copyfile("%s/settings.xml" % path_system, "%s/settings.xml" % path_user)
#            copyfile("%s/symbols/symbols.xml" % path_system, "%s/symbols/symbols.xml" % path_user)
#        except IOError:
#            error(self, "__init__", "Failed to copy a settings file to user directory")
#            
#elif exists(path_user):
#    debug(None, "", "Detected user installation")
#    install_path = path_user
#    
#else:
#    error(self, "__init__", "Failed to determine the type of installation (user/system)")
#
#settings_filename = "%s/settings.xml" % path_user
#symbols_filename = "%s/symbols/symbols.xml" % path_user



class Installation(object):
	"""
	This encapsulates the plugin's installation settings
	"""
	def __new__(type):
		if not '_instance' in type.__dict__:
			type._instance = object.__new__(type)
		return type._instance
	
	def __init__(self):
		if not '_ready' in dir(self):
			
			###
			
			self._path_system = "/usr/lib/gedit-2/plugins/LaTeXPlugin"
			self._path_user = expanduser("~/.gnome2/gedit/plugins/LaTeXPlugin")
			
			if exists(self._path_system):
			    debug(self, "__init__", "Detected system-wide installation")
			    self._install_path = self._path_system
			    
			    if not exists(self._path_user):
			        # copy settings.xml to user dir
			        makedirs(self._path_user)
			        
			        try:
			            copyfile("%s/settings.xml" % self._path_system, "%s/settings.xml" % self._path_user)
			            copyfile("%s/symbols/symbols.xml" % self._path_system, 
								"%s/symbols/symbols.xml" % self._path_user)
			        except IOError:
			            error(self, "__init__", "Failed to copy a settings file to user directory")
			elif exists(self._path_user):
			    debug(self, "__init__", "Detected user installation")
			    self._install_path = self._path_user
			    
			else:
			    error(self, "__init__", "Failed to determine the type of installation (user/system)")
			
			self._settings_filename = "%s/settings.xml" % self._path_user
			self._symbols_filename = "%s/symbols/symbols.xml" % self._path_user
			
			###
			
			self._ready = True
	
	@property
	def path(self):
		return self._install_path
	
	@property
	def settings_filename(self):
		return self._settings_filename
	
	@property
	def symbols_filename(self):
		return self._symbols_filename
    