# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Sdocumentt, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import TreeStore
from gtk.gdk import Pixbuf, pixbuf_new_from_file
from xml.sax.saxutils import escape

from LaTeXPlugin.Installation import Installation

from BibTeXModel import BibTeXModel
from BibTeXEntry import BibTeXEntry

from LaTeXPlugin.outline import OutlineNodePosition

class BibTeXOutlineModel(BibTeXModel, TreeStore):
	"""
	This is passed to the BibTeXParser, filled with entries and then passed an OutlineView to be
	displayed.
	"""
	
	# TOOD: icon registry
	
	__pb_entry = pixbuf_new_from_file(Installation().path + "/pixmaps/documents.png")
	__pb_field = pixbuf_new_from_file(Installation().path + "/pixmaps/field.png")
	
	def __init__(self):
		BibTeXModel.__init__(self)
		TreeStore.__init__(self, str, Pixbuf, object)
	
	def add_entry(self, entry):
		if type(entry) is BibTeXEntry:
			# append entry
			parent = self.append(None, ["%s <small>%s</small>" % (escape(entry.key), escape(entry.type)), 
									self.__pb_entry, OutlineNodePosition(entry.line, entry.column)])
			
			# append fields
			for name, value in entry.fields.iteritems():
				self.append(parent, ["<small><b>%s</b></small> %s" % (escape(name), escape(str(value))), 
									self.__pb_field, OutlineNodePosition(entry.line, entry.column)])
	
	def clear(self):
		BibTeXModel.clear(self)
		TreeStore.clear(self)