# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

class StringReader(object):
	def __init__(self, string):
		self.__iter = iter(string)
		
		self.__eof = False
		self.__line = 1
		self.__column = 0
		self.__current = None
		
		self.step()
	
	def step(self):
		if self.__eof:
			return
		
		try:
			self.__current = self.__iter.next()
			if self.__current == "\n":
				self.__line += 1
				self.__column = 0
			else:
				self.__column += 1
		except StopIteration:
			self.__eof = True
	
	@property
	def line(self):
		return self.__line
	
	@property
	def column(self):
		return self.__column
	
	@property
	def eof(self):
		return self.__eof
	
	@property
	def current(self):
		return self.__current