# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import RESPONSE_OK

from BuildImageDialog import BuildImageDialog, PNG, JPG, GIF
from Profile import Profile
from Job import Job

class ImageProfileGenerator(object):
	"""
	This generates a build profile from the settings made in the BuildImageDialog
	"""
	def __init__(self):
		self.__dialog = None
		
	def run(self):
		if not self.__dialog:
			self.__dialog = BuildImageDialog()
		
		profile = None
		
		if self.__dialog.run() == RESPONSE_OK:
			
			format = self.__dialog.get_format()
			png_mode = self.__dialog.get_png_mode()
			render_box = self.__dialog.get_box_enabled()
			resolution = self.__dialog.get_resolution()
			aa_factor = self.__dialog.get_aa_factor()
			
			profile = Profile()
			profile.name = { PNG : "PNG", JPG : "JPEG", GIF : "GIF" }[format] + "-Image"
			
			# rubber (dvi)
			job = Job()
			job.command = "rubber -f -s --inplace %f"
			job.stopper = True
			job.processor = "rubber"
			profile.jobs.append(job)
			
			if render_box:
				# dvips
				job = Job()
				job.command = "dvips -D %s -q -E -o %%e.eps %%e.dvi" % resolution
				job.stopper = True
				job.processor = "default"
				profile.jobs.append(job)
				
				# eps2png
				#
				# TODO: Processor for eps2png
				#
				job = Job()
				job.stopper = True
				job.processor = "default"
				
				if format == PNG:
					job.command = "%%p/util/eps2png -png%s -resolution=%s -aa=%s %%e.eps" % (png_mode, resolution, aa_factor)
				elif format == JPG:
					job.command = "%%p/util/eps2png -jpeg -resolution=%s -aa=%s %%e.eps" % (resolution, aa_factor)
				elif format == GIF:
					job.command = "%%p/util/eps2png -gif -resolution=%s -aa=%s %%e.eps" % (resolution, aa_factor)
					
				profile.jobs.append(job)
			else:
				# dvips
				job = Job()
				job.stopper = True
				job.processor = "default"
				job.command = "dvips -D %s -q -o \"%%e.ps\" \"%%e.dvi\"" % resolution
				profile.jobs.append(job)
							
				# gs
				job = Job()
				job.stopper = True
				job.processor = "default"
				
				if format == PNG:
					job.command = "gs -q -dNOPAUSE -r%s -dTextAlphaBits=%s -dGraphicsAlphaBits=%s -sDEVICE=png%s -sOutputFile=%%e.png %%e.ps quit.ps" % (resolution, aa_factor, aa_factor, png_mode)
					profile.jobs.append(job)
				elif format == JPG:
					job.command = "gs -q -dNOPAUSE -r%s -dTextAlphaBits=%s -dGraphicsAlphaBits=%s -sDEVICE=jpeg -sOutputFile=%%e.jpg %%e.ps quit.ps" % (resolution, aa_factor, aa_factor)
					profile.jobs.append(job)
				elif format == GIF:
					job.command = "gs -q -dNOPAUSE -r%s -dTextAlphaBits=%s -dGraphicsAlphaBits=%s -sDEVICE=ppm -sOutputFile=%%e.ppm %%e.ps quit.ps" % (resolution, aa_factor, aa_factor)
					profile.jobs.append(job)
					
					# ppmtogif
					job = Job()
					job.stopper = True
					job.processor = "default"
					job.command = "ppmtogif %e.ppm > %e.gif"
					profile.jobs.append(job)
			
			profile.viewer = "gnome-open %e" + { PNG:".png", JPG:".jpg", GIF:".gif" }[format]
		
		self.__dialog.hide()
		return profile
		
		