# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Dialog, DIALOG_DESTROY_WITH_PARENT, DIALOG_MODAL, STOCK_CANCEL, RESPONSE_CANCEL, \
	STOCK_OK, RESPONSE_OK, FileChooserDialog, FILE_CHOOSER_ACTION_OPEN, Image, Entry, Button, \
	STOCK_OPEN, HButtonBox, BUTTONBOX_END, VBox, Alignment, Frame, SHADOW_NONE, SpinButton, \
	Label, ToggleButton, RELIEF_NONE, Table, combo_box_new_text, CheckButton, FILL, FileChooserButton, \
	RESPONSE_ACCEPT, RadioButton, Adjustment, Alignment
from gtk.gdk import pixbuf_new_from_file_at_size, pixbuf_new_from_file

from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.UnitComboBox import UnitComboBox, PERCENT, PIXEL

from GraphicsSourceBuilder import GraphicsSourceBuilder

class GraphicsDialog(Dialog):
	def __init__(self, parent):
		Dialog.__init__(self, "Insert Graphics", parent, DIALOG_DESTROY_WITH_PARENT|DIALOG_MODAL, \
					(STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		self.set_resizable(False)
		
		self.__builder = None
		self.__enabled = False
		
		# file
		self.dlg_file = FileChooserDialog(action=FILE_CHOOSER_ACTION_OPEN, 
										buttons=(STOCK_OPEN, RESPONSE_ACCEPT, STOCK_CANCEL, RESPONSE_CANCEL))
		img_preview = Image()
		self.dlg_file.set_preview_widget(img_preview)
		self.dlg_file.set_local_only(True)
		self.dlg_file.connect("update-preview", self.__update_preview, img_preview)
		self.dlg_file.connect("response", self.__file_response)
		
		self.btn_file = FileChooserButton(self.dlg_file)
		
		self.lbl_file = Label("<b>_File</b>")
		self.lbl_file.set_alignment(0, .5)
		self.lbl_file.set_use_markup(True)
		self.lbl_file.set_use_underline(True)
		self.lbl_file.set_mnemonic_widget(self.btn_file)
		
		self.lbl_original_size = Label()
		self.lbl_original_size.set_alignment(0, .5)
		
		# resize
		lbl_resize = Label("<b>Resize</b>")
		lbl_resize.set_alignment(0, .5)
		lbl_resize.set_use_markup(True)
		
		self.spn_width = SpinButton(Adjustment(0, 0, 100000, 1))
		self.spn_width.set_activates_default(True)
		
		self.spn_height = SpinButton(Adjustment(0, 0, 100000, 1))
		self.spn_height.set_activates_default(True)
		
		lbl_height = Label("_Height:")
		lbl_height.set_alignment(0, .5)
		lbl_height.set_use_underline(True)
		lbl_height.set_mnemonic_widget(self.spn_height)
		
		lbl_width = Label("_Width:")
		lbl_width.set_alignment(0, .5)
		lbl_width.set_use_underline(True)
		lbl_width.set_mnemonic_widget(self.spn_width)
		
		self.cmb_unit = UnitComboBox()
		self.cmb_unit.attach_spinbutton(self.spn_width)
		self.cmb_unit.attach_spinbutton(self.spn_height)
		self.cmb_unit.set_enable_percent(True)
		self.cmb_unit.set_unit(PERCENT)
		
		img_lock = Image()
		img_lock.set_from_file(Installation().path + "/pixmaps/lock.png")
		
		self.btn_lock = ToggleButton(None)
		self.btn_lock.set_relief(RELIEF_NONE)
		self.btn_lock.add(img_lock)
		self.btn_lock.connect("toggled", self.__lock_toggled)
		self.btn_lock.set_active(True)

		# transform
		lbl_transform = Label("<b>Transform</b>")
		lbl_transform.set_alignment(0, .5)
		lbl_transform.set_use_markup(True)
		
		self.spn_rotate = SpinButton(Adjustment(0, -360, 360, 1))
		self.spn_rotate.set_activates_default(True)
		self.spn_rotate.set_digits(2)
		
		lbl_rotate = Label("_Rotate:")
		lbl_rotate.set_alignment(0, .5)
		lbl_rotate.set_use_underline(True)
		lbl_rotate.set_mnemonic_widget(self.spn_rotate)
		
		self.chk_reflect = CheckButton("Reflect h_orizontally", True)
		
		# floating
		lbl_float = Label("<b>Floating</b>")
		lbl_float.set_alignment(0, .5)
		lbl_float.set_use_markup(True)
		
		self.chk_float = CheckButton("Insert as _floating object", True)
		self.chk_float.set_active(True)
		self.chk_float.connect("toggled", self.__float_toggled)
		
		self.lbl_placement = Label("Placement:")
		self.lbl_placement.set_alignment(0, .5)
		
		self.chk_placement_h = CheckButton("Where _defined", True)
		self.chk_placement_t = CheckButton("_Top of page", True)
		self.chk_placement_b = CheckButton("_Bottom of page", True)
		self.chk_placement_p = CheckButton("Separate _page", True)
		
		self.tbl_placement = Table()
		self.tbl_placement.attach(self.chk_placement_h, 0, 1, 0, 1)
		self.tbl_placement.attach(self.chk_placement_t, 1, 2, 0, 1)
		self.tbl_placement.attach(self.chk_placement_b, 0, 1, 1, 2)
		self.tbl_placement.attach(self.chk_placement_p, 1, 2, 1, 2)
		
		self.ent_caption = Entry()
		self.ent_caption.set_activates_default(True)
		
		self.lbl_caption = Label("_Caption:")
		self.lbl_caption.set_alignment(0, .5)
		self.lbl_caption.set_use_underline(True)
		self.lbl_caption.set_mnemonic_widget(self.ent_caption)
		
		self.ent_label = Entry()
		self.ent_label.set_activates_default(True)
		
		self.lbl_label = Label("_Label:")
		self.lbl_label.set_alignment(0, .5)
		self.lbl_label.set_use_underline(True)
		self.lbl_label.set_mnemonic_widget(self.ent_label)

		self.tbl = Table()
		
		self.tbl.attach(self.lbl_file, 0, 5, 0, 1)
		self.tbl.attach(self.btn_file, 1, 5, 1, 2, yoptions=0)
		self.tbl.attach(self.lbl_original_size, 1, 5, 2, 3)
		self.tbl.attach(lbl_resize, 0, 5, 3, 4)
		self.tbl.attach(lbl_width, 1, 2, 4, 5)
		self.tbl.attach(self.spn_width, 2, 3, 4, 5)
		self.tbl.attach(lbl_height, 1, 2, 5, 6)
		self.tbl.attach(self.spn_height, 2, 3, 5, 6)
		self.tbl.attach(self.btn_lock, 3, 4, 4, 6)
		self.tbl.attach(self.cmb_unit, 4, 5, 5, 6, yoptions=0)
		self.tbl.attach(lbl_transform, 0, 5, 6, 7)
		self.tbl.attach(lbl_rotate, 1, 2, 7, 8)
		self.tbl.attach(self.spn_rotate, 2, 3, 7, 8)
		self.tbl.attach(self.chk_reflect, 1, 5, 8, 9)
		self.tbl.attach(lbl_float, 0, 5, 9, 10)
		self.tbl.attach(self.chk_float, 1, 5, 10, 11)
		self.tbl.attach(self.lbl_placement, 1, 2, 11, 12, yoptions=0)
		self.tbl.attach(self.tbl_placement, 2, 5, 11, 13)
		self.tbl.attach(self.lbl_caption, 0, 2, 13, 14)
		self.tbl.attach(self.ent_caption, 2, 5, 13, 14)
		self.tbl.attach(self.lbl_label, 0, 2, 14, 15)
		self.tbl.attach(self.ent_label, 2, 5, 14, 15)
		
		self.tbl.set_border_width(5)
		self.tbl.set_row_spacings(5)
		self.tbl.set_col_spacings(5)
		
		self.tbl.set_col_spacing(0, 15)
		self.tbl.set_row_spacing(4, 0)
		self.tbl.set_row_spacing(2, 15)
		self.tbl.set_row_spacing(5, 15)
		self.tbl.set_row_spacing(8, 15)
		
		self.tbl.show_all()
		
		self.vbox.add(self.tbl)
		
		self.set_default_response(RESPONSE_OK)
	
	def __float_toggled(self, togglebutton):
		v = togglebutton.get_active()
		
		self.lbl_placement.set_sensitive(v)
		self.tbl_placement.set_sensitive(v)
		#self.lbl_caption.set_sensitive(v)
		#self.ent_caption.set_sensitive(v)
		#self.lbl_label.set_sensitive(v)
		#self.ent_label.set_sensitive(v)
	
	def __lock_toggled(self, togglebutton):
		self.cmb_unit.set_keep_aspect_ratio(togglebutton.get_active())
	
	def __file_response(self, dialog, response):
		if response == RESPONSE_ACCEPT:
			filename = dialog.get_filename()
			
			self.cmb_unit.set_unit(PERCENT)
			self.spn_width.set_value(100)
			self.spn_height.set_value(100)
			
			try:
				# load the image to a pixbuf
				pixbuf = pixbuf_new_from_file(filename)
				
				# store original size and aspect ratio
				self.size = (pixbuf.get_width(), pixbuf.get_height())
				
				# show the image's original size
				self.lbl_original_size.set_markup("<small>%s x %s Pixels</small>" % (self.size[0], self.size[1]))
				
				self.cmb_unit.set_initial(self.spn_width, pixbuf.get_width(), PIXEL)
				self.cmb_unit.set_initial(self.spn_height, pixbuf.get_height(), PIXEL)
			except:
				pass
			
			self.__set_enabled(True)
	
	def __update_preview(self, filechooser, preview):
		""" 
		Update the FileChooser's preview image 
		"""
		filename = filechooser.get_preview_filename()
		try:
			pixbuf = pixbuf_new_from_file_at_size(filename, 128, 128)
			preview.set_from_pixbuf(pixbuf)
			have_preview = True
		except:
			have_preview = False
		filechooser.set_preview_widget_active(have_preview)
	
	def __reset(self):
		"""
		Reset the dialog
		"""
		self.dlg_file.unselect_all()
		self.spn_width.set_value(0)
		self.spn_height.set_value(0)
		self.lbl_original_size.set_text("")
		self.ent_caption.set_text("")
		self.ent_label.set_text("")
		self.chk_float.set_active(True)
		self.chk_reflect.set_active(False)
		self.spn_rotate.set_value(0)
		
		self.__set_enabled(False)
	
	def __set_enabled(self, enabled):
		for child in self.tbl.get_children():
			if child not in [self.lbl_file, self.btn_file]:
				child.set_sensitive(enabled)
		
		if enabled:
			self.lbl_original_size.show()
		else:
			self.lbl_original_size.hide()
		
		self.__enabled = enabled
	
	def run(self, working_directory):
		self.__reset()
		self.__working_directory = working_directory
		
		self.btn_file.grab_focus()
		
		return Dialog.run(self)
	
	@property
	def source(self):
		if not self.__builder:
			self.__builder = GraphicsSourceBuilder()
		
		self.__builder.working_directory = self.__working_directory
		self.__builder.filename = self.dlg_file.get_filename()
		
		if self.cmb_unit.get_unit() == PERCENT:
			self.__builder.h_scale = self.spn_width.get_value() / 100.0
			self.__builder.v_scale = self.spn_height.get_value() / 100.0
		else:
			self.__builder.resize_width = self.spn_width.get_value()
			self.__builder.resize_height = self.spn_height.get_value()
			self.__builder.resize_unit = self.cmb_unit.get_unit_string()
		
		self.__builder.rotate = self.spn_rotate.get_value()
		self.__builder.reflect = self.chk_reflect.get_active()
		
		self.__builder.float = self.chk_float.get_active()
		
		if self.chk_float.get_active():
			self.__builder.placement_b = self.chk_placement_b.get_active()
			self.__builder.placement_h = self.chk_placement_h.get_active()
			self.__builder.placement_t = self.chk_placement_t.get_active()
			self.__builder.placement_p = self.chk_placement_p.get_active()
			
		self.__builder.caption = self.ent_caption.get_text()
		self.__builder.label = self.ent_label.get_text()
		
		return self.__builder.get_source()
	